@@ -34,6 +34,7 @@ this function can call (this is not an exhaustive list):
     - govukInput
     - govukLabel
     - govukRadios
+    - govukFileUpload
     - dmListInput
 
 `render_question()` should cover all the usual cases of creating a form from a
@@ -45,7 +46,7 @@ Read the docstring for `from_question()` for more detail on how Questions are
 handled.
 """
 
-from typing import List, Optional, TYPE_CHECKING
+from typing import List, Optional, Set, TYPE_CHECKING
 
 import jinja2
 from jinja2 import Markup, escape
@@ -135,6 +136,11 @@ def from_question(
             "macro_name": "govukCharacterCount",
             "params": govuk_character_count(question, data, errors, **kwargs)
         }
+    elif question.type == "upload":
+        return {
+            "macro_name": "govukFileUpload",
+            "params": govuk_file_upload(question, data, errors, **kwargs)
+        }
     elif question.type == "multiquestion":
         return dm_multiquestion(question, data, errors, **kwargs)
     else:
@@ -235,10 +241,11 @@ def govuk_radios(
     params["idPrefix"] = f"input-{question.id}"
 
     if question.get("type") == "boolean":
-        if params.get("classes"):
-            params["classes"] += " govuk-radios--inline"
-        else:
-            params["classes"] = "govuk-radios--inline"
+        if "followup" not in question:
+            if params.get("classes"):
+                params["classes"] += " govuk-radios--inline"
+            else:
+                params["classes"] = "govuk-radios--inline"
         options = [{"label": "Yes", "value": "True"}, {"label": "No", "value": "False"}]
         params["items"] = govuk_options(options, str(data.get(question.id)))
     else:
@@ -247,6 +254,36 @@ def govuk_radios(
     return params
 
 
+def govuk_file_upload(
+        question: 'Question', data: Optional[dict] = None, errors: Optional[dict] = None, **kwargs
+) -> dict:
+    """Create govukFileUpload macro parameters from a file upload question"""
+    params = _params(question, data, errors, **kwargs)
+
+    params["label"] = {"text": question.question}
+
+    params["hint"] = {"html": ""}
+    if question.get("question_advice"):
+        params["hint"]["html"] += question.get("question_advice")
+
+    if question.get("hint"):
+        params["hint"]["html"] += question.get("hint")
+
+    # If the user has previously uploaded a file for this question
+    # add a line to the hint text rather than pre-filling the input
+    if data and data.get(params["name"]):
+        params["hint"]["html"] += Markup("<p>Previously uploaded file:<br>")
+        params["hint"]["html"] += Markup(data.get(params["name"]))
+        params["hint"]["html"] += Markup("</p>")
+
+    # Set an empty key in params for `question_advice` so `render` doesn't
+    # add the advice again.
+    # TODO: remove this once `render` doesn't check for question advice
+    params["question_advice"] = ""
+
+    return params
+
+
 def dm_list_input(
     question: 'Question', data: Optional[dict] = None, errors: Optional[dict] = None, **kwargs
 ) -> dict:
@@ -315,10 +352,39 @@ def dm_multiquestion(
     if question.get("question_advice"):
         to_render.append(_question_advice(question))
 
-    to_render += [
-        from_question(q, data, errors, is_page_heading=False)
-        for q in question.questions
-    ]
+    # We need to be able to skip followup questions. Questions which have
+    # followups always come before the followup, so we create a variable
+    # outside of the loop scope which is used to flag when a subsequent
+    # question should be skipped. Followups are referred to by id in the
+    # question object, so our variable should be a set of ids.
+    to_skip: Set[str] = set()
+
+    for q in question.questions:
+        if q.id in to_skip:
+            continue
+
+        to_render.append(
+            from_question(q, data, errors, is_page_heading=False)
+        )
+
+        if q.get("followup"):
+            # flag that the followup question(s) should be skipped later
+            to_skip.update(q.followup.keys())
+
+            # convert the values in values_followup to str
+            # to match the form input item values
+            followups = {str(v): qs for v, qs in q.values_followup.items()}
+            items = to_render[-1]["params"]["items"]
+
+            for item in items:
+                if item["value"] in followups:
+                    followup_items = [
+                        from_question(question.get_question(followup_id), data, errors, is_page_heading=False)
+                        for followup_id in followups[item["value"]]
+                    ]
+                    item["conditional"] = {
+                        "html": followup_items
+                    }
 
     return to_render
 
@@ -526,11 +592,7 @@ def render(ctx, obj, *, question=None) -> Markup:
                 def visit(inner_obj):
                     if isinstance(inner_obj, dict):
                         for k, v in inner_obj.items():
-                            if (
-                                k == "html"
-                                and isinstance(v, dict)
-                                and "macro_name" in v
-                            ):
+                            if k == "html":
                                 inner_obj["html"] = render(ctx, v)
                             else:
                                 visit(v)
